require 'morq/base'
require 'socket'
require 'fileutils'

module Morq::Maildir
  class Error<Exception
  end

  def store(input, dir, tmpdir)
    # 1) check two directories
    dev =[]
    for d in [dir, tmpdir]
      stat = File.stat(d)
      unless stat.directory?
        raise Error, format("%s not a directory", d)
      end
      unless stat.writable?
        raise Error, format("%s not writable", d)
      end
      dev.push stat.dev
    end
    unless dev[0] == dev[1]
      raise Error, format("%s and %s is not on same device", dir, tmp)
    end
    # 2) get the fname
    host = Socket.gethostname
    fname = nil
    loop do
      now = Time.now
      fname = format("%d%d.%d.%s", now.to_i, now.usec, Process.pid, host)
      # 3) retry if fname exists already
      break unless File.exist? fname
    end
    # 4) open the file in the tmpdir
    tmp = File.join(tmpdir, fname)
    begin
      open(tmp, "w") do |f|
        FileUtils.copy_stream(input, f)
      end
      File.link(tmp, File.join(dir, fname))
    ensure
      File.unlink(tmp)
    end
    fname
  end
  module_function :store
end

if $0 == __FILE__
  begin
    puts Morq::Maildir::store(STDIN, ARGV[0], ARGV[1])
  rescue
    exit 1
  end
end
