require 'morq/base'
require 'morq/list'
require 'rast'

# LIST_QUERY(query): done - score.rb
# - get qresult... from query using inverted index
# - LIST_INDEX(qresult..., false)

# LIST_SIMILARITY(mid): - score.rb
# - get most significant words from mid using inverted index
# - get qresult... from words using inverted index
# - trim qresult... using score
# - LIST_INDEX(qresult..., thread=false)

module Morq::Query
  include Morq::List

  private

  def create_inverted_index()
    properties = [
      {
        "name" => "filename",
        "type" => Rast::PROPERTY_TYPE_STRING,
        "search" => true,
        "text_search" => false,
        "full_text_search" => false,
        "unique" => true,
      },
      {
        "name" => "subject",
        "type" => Rast::PROPERTY_TYPE_STRING,
        "search" => false,
        "text_search" => true,
        "full_text_search" => true,
        "unique" => false,
      },
      {
        "name" => "from",
        "type" => Rast::PROPERTY_TYPE_STRING,
        "search" => false,
        "text_search" => true,
        "full_text_search" => true,
        "unique" => false,
      },
      {
        "name" => "date",
        "type" => Rast::PROPERTY_TYPE_DATE,
        "search" => true,
        "text_search" => false,
        "full_text_search" => false,
        "unique" => false,
      },
      {
        "name" => "mid",
        "type" => Rast::PROPERTY_TYPE_UINT,
        "search" => true,
        "text_search" => false,
        "full_text_search" => false,
        "unique" => true,
      },
    ]

    options = {
      "byte_order" => Rast::LITTLE_ENDIAN,
      "encoding" => "euc_jp",
      "preserve_text" => false,
      "properties" => properties,
    }
    Rast::DB.create(morq_file("rast"), options)
  end

  def open_inverted_index(update=false)
    mode = if update then Rast::DB::RDWR else Rast::DB::RDONLY end
    unless File.exist?(morq_file("rast"))
      create_inverted_index()
    end
    Rast::DB.open(morq_file("rast"), mode) do |db|
      yield db
    end
  end

  def add_inverted_index(text, properties, extern=nil)
    begin
      if extern
        extern.register(text, properties)
      else
        open_inverted_index(:update) do |db|
          db.register(text, properties)
        end
      end
    rescue Rast::RastError
      # ignore already registered file
    end
    print properties['date'], " [",  properties['from'], "] ",  properties['subject'], "\n" if $DEBUG
  end

  def query_result(query)
    options = {
      "num_items" => Rast::RESULT_ALL_ITEMS,
      "properties" => ["mid"],
    }

    data = ""
    query.gsub!(/limit\s*=\s*(\d+)/) do
      options["num_items"] = $1.to_i
      ""
    end
    query.gsub!(/start\s*=\s*(\d+)/) do
      options["start_no"] = $1.to_i
      ""
    end
    labels = nil
    query.gsub!(/label\s*=\s*(\w+)/) do
      labels ||= []
      labels.push $1
      ""
    end
    open_inverted_index() do |db|
      result = db.search(query, options)
      if labels
        open_label_db() do |db|
          result.items.each do |item|
            mid = item.properties[0]
            common = labels & db.get_labels(mid)
            if common.size == labels.size
              data.concat pack_id(mid)
            end
          end
        end
      else
        result.items.each do |item|
          data.concat pack_id(item.properties[0])
        end
      end
    end
    return data
  end

  def mid_doc_id(db, mid)
    mid = ARGV[0].to_i
    result = db.search("mid = %d"%mid,
                       "num_items" => 1,
                       "properties" => ["mid"])
    result.items.each do |item|
      STDERR.puts item.properties[0]
      return item.properties[0]
    end
    0
  end

  def delete_inverted_index(mid)
    open_inverted_index(:update) do |db|
      doc_id = mid_doc_id(db, mid)
      db.delete(doc_id) unless doc_id == 0 rescue 0
    end
  end

end

if __FILE__ == $0
  include Morq::Query
  
  box_query((ARGV[0] || "matz")).list(STDOUT)  
#  p eid_from_mid(1)
#box_query("").list(STDOUT)
#  box_query("ڥ").list(STDOUT)
#  box_query("soft").list(STDOUT)
#  box_similarity(1).list(STDOUT)
end
